/*
  Softshell: Dynamic Scheduling on GPUs.
  http://www.icg.tugraz.at/project/mvp

  Copyright (C) 2012 Institute for Computer Graphics and Vision,
                     Graz University of Technology

  Author(s):  Markus Steinberger - steinberger ( at ) icg.tugraz.at
              Bernhard Kainz - kainz ( at ) icg.tugraz.at
              Michael Kenzel - kenzel ( at ) icg.tugraz.at
              Stefan Hauswiesner - hauswiesner ( at ) icg.tugraz.at
              Bernhard Kerbl - kerbl ( at ) icg.tugraz.at
              Dieter Schmalstieg - schmalstieg ( at ) icg.tugraz.at

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
*/


/*
* file created by    Markus Steinberger / steinberger ( at ) icg.tugraz.at
*
* modifications by
*/

#ifndef SOFTSHELL_MEMORY_MAPPEDMEMORY_INCLUDED
#define SOFTSHELL_MEMORY_MAPPEDMEMORY_INCLUDED

#include "tools/types.h"
#include "tools/thread.h"
#include <utility>


namespace Softshell
{
  class MappedMemory
  {
    struct MemInfo
    {
      bool used;
      uint size;
      int offset;
      MemInfo *prev, *next;
      
      MemInfo(int _offset, uint _size, MemInfo* _prev = 0, MemInfo* _next = 0, bool _used = false);
      int alloc(uint _size);
      void free();
    };
    MemInfo* begin;

    unsigned char* h_memory, *d_memory;
    Mutex mutex;
    size_t memorysize;

    int allocOffset(uint size);
    void free(int p);
    void* d_offsetToPointer(int offset) const;
    void* h_offsetToPointer(int offset) const;

  public:
    MappedMemory(size_t _memorysize = 2U*1024U*1024U);

    std::pair<void*,void*> alloc(uint size);
    void free(std::pair<void*,void*> ppair);
    void free(void* p);
    void* getDevicePointer(void* hostpointer) const;
    void* getHostPointer(void* devicepointer) const;
  };
};

#endif //SOFTSHELL_MEMORY_MAPPEDMEMORY_INCLUDED