/*
  Softshell: Dynamic Scheduling on GPUs.
  http://www.icg.tugraz.at/project/mvp

  Copyright (C) 2012 Institute for Computer Graphics and Vision,
                     Graz University of Technology

  Author(s):  Markus Steinberger - steinberger ( at ) icg.tugraz.at
              Bernhard Kainz - kainz ( at ) icg.tugraz.at
              Michael Kenzel - kenzel ( at ) icg.tugraz.at
              Stefan Hauswiesner - hauswiesner ( at ) icg.tugraz.at
              Bernhard Kerbl - kerbl ( at ) icg.tugraz.at
              Dieter Schmalstieg - schmalstieg ( at ) icg.tugraz.at

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
*/

#include "softshell.h"

#include <stdio.h>

#include "proc1.cuh"
#include "proc2.cuh"

using namespace Softshell;

void runtest(int device)
{
  //create the scheduler for the specified device only
  Scheduler scheduler(device);
  //scheduler.setConfigParam("SimpleMegakernel",1U);
  scheduler.setConfigParam("SeparateController",1U);

  //create an event which should create BlockWorkpackage for the MyProc1 procedure

  // on Windows you can also do that...
  //auto myevent = scheduler.createEvent<BlockWorkpackage, MyProc1>();
  PEvent myevent = scheduler.createEvent<BlockWorkpackage, MyProc1>();

  //fire it and let it create 1 workpackage (with id 0,0,0), which should be executed by 32 threads
  uint id0 = myevent->fire(DefaultCreatorParams<>(4, 32));
  //wait for the event to be finished
  (*myevent).join();

  cudaDeviceSynchronize();

  //optional shut down
  scheduler.shutdown();
}
